@@ -159,6 +159,10 @@ setReplaceMethod("[", signature(x = "CsparseMatrix", i = "index", j = "index",
 
 setMethod("crossprod", signature(x = "CsparseMatrix", y = "missing"),
 	  function(x, y = NULL) {
+              if (is(x, "symmetricMatrix")) {
+                  warning("crossprod(x) calculated as x %*% x for sparse, symmetric x")
+                  return(x %*% x)
+              }
 	      .Call(Csparse_crossprod, x, trans = FALSE, triplet = FALSE)
 	  })
 
@@ -175,6 +179,10 @@ setMethod("crossprod", signature(x = "CsparseMatrix", y = "numeric"),
 
 setMethod("tcrossprod", signature(x = "CsparseMatrix", y = "missing"),
 	  function(x, y = NULL) {
+              if (is(x, "symmetricMatrix")) {
+                  warning("tcrossprod(x) calculated as x %*% x for sparse, symmetric x")
+                  return(x %*% x)
+              }
               .Call(Csparse_crossprod, x, trans = TRUE, triplet = FALSE)
 	  })
 
@@ -252,21 +260,26 @@ setMethod("band", "CsparseMatrix",
 	  })
 
 setMethod("diag", "CsparseMatrix",
-          function(x, nrow, ncol = n) {
-              dm <- .Call(Csparse_band, x, 0, 0)
-              dlen <- min(dm@Dim)
-              ind1 <- dm@i + 1:1        # 1-based index vector
-              if (is(dm, "lMatrix")) {
-                  val <- rep.int(FALSE, dlen)
-                  val[ind1] <- TRUE
-                  return(val)
-              }
-              val <- rep.int(0, dlen)
-              ## cMatrix not yet active but for future expansion
-              if (is(dm, "cMatrix")) val <- as.complex(val)
-              val[ind1] <- dm@x
-              val
-          })
+	  function(x, nrow, ncol = n) {
+	      dm <- .Call(Csparse_band, x, 0, 0)
+	      dlen <- min(dm@Dim)
+	      ind1 <- dm@i + 1:1	# 1-based index vector
+	      if (is(dm, "nMatrix")) {
+		  val <- rep.int(FALSE, dlen)
+		  val[ind1] <- TRUE
+	      }
+	      else if (is(dm, "lMatrix")) {
+		  val <- rep.int(FALSE, dlen)
+		  val[ind1] <- as.logical(dm@x)
+	      }
+	      else {
+		  val <- rep.int(0, dlen)
+		  ## cMatrix not yet active but for future expansion
+		  if (is(dm, "cMatrix")) val <- as.complex(val)
+		  val[ind1] <- dm@x
+	      }
+	      val
+	  })
 
 
 setMethod("colSums", signature(x = "CsparseMatrix"), .as.dgC.Fun,
