@@ -4,14 +4,7 @@
 setAs("dMatrix", "matrix",
       function(from) as(as(from, "dgeMatrix"), "matrix"))
 
-### Idea: Coercion between *VIRTUAL* classes -- as() chooses "closest" classes
-### ----  should also work e.g. for  triangular --> triangular !
-##
-## setAs("dMatrix", "lMatrix",
-##       function(from) {
-##       })
-
-##-> this is now in ./Matrix.R
+##-> "dMatrix" <--> "lMatrix"   ---> ./lMatrix.R
 
 ## Methods for operations where one argument is integer
 ## No longer made use of (and confusing hence) since R version 2.1.0
@@ -81,69 +74,73 @@ setMethod("Compare", signature(e1 = "numeric", e2 = "dMatrix"),
 
 setMethod("Compare", signature(e1 = "dMatrix", e2 = "numeric"),
 	  function(e1, e2) {
-              lClass <- class2(class(e1), "l")
-              fullCl <- if(isSymmetric(e1)) "lsyMatrix" else "lgeMatrix"
+	      lClass <- class2(class(e1), "l")
+	      fullCl <- if(isSymmetric(e1)) "lsyMatrix" else "lgeMatrix"
 	      ## Dbg cat("Compare", class(e1), "|-> ",lClass, "\n")
-	      r  <- callGeneric(e1@x, e2)
-              r0 <- callGeneric(0, e2)
-              ## trivial case first
-              if(isTRUE(r0) && all(r)) {
-                  r <- new(fullCl)
-                  d <- e1@Dim
-                  r@Dim <- d
-                  r@Dimnames <- e1@Dimnames
-                  r@x <- rep.int(TRUE, prod(d))
-              }
-	      else if(is(e1, "denseMatrix")) {
-                  full <- !isPacked(e1) # << both "dtr" and "dsy" are 'full'
-                  if(full || identical(r0, FALSE) || is(e1, "symmetricMatrix"))
-                      r <- new(lClass, x = r,
-                               Dim = dim(e1), Dimnames = dimnames(e1))
-                  else { ## packed matrix with structural 0 and r0 is not FALSE:
-                      ##--> result cannot be packed anymore
-                      .bail.out.2(.Generic, class(e1), class(e2))
-                      dr <- as(r, fullCl)
-                      ## FIXME: implement this:
-                      dr[ind.0(e1)] <- r0
-                  }
+	      r	 <- callGeneric(e1@x, e2)
+	      r0 <- callGeneric(0, e2)
+	      ## trivial case first
+	      if(isTRUE(r0) && all(r)) {
+		  r <- new(fullCl)
+		  d <- e1@Dim
+		  r@Dim <- d
+		  r@Dimnames <- e1@Dimnames
+		  r@x <- rep.int(TRUE, prod(d))
 	      }
-              else { ## dsparseMatrix => lClass is "lsparse*"
+	      else if(is(e1, "denseMatrix")) {
+		  full <- !isPacked(e1) # << both "dtr" and "dsy" are 'full'
+		  if(full || identical(r0, FALSE) || is(e1, "symmetricMatrix"))
+		      r <- new(lClass, x = r,
+			       Dim = dim(e1), Dimnames = dimnames(e1))
+		  else { ## packed matrix with structural 0 and r0 is not FALSE:
+		      ##--> result cannot be packed anymore
+		      .bail.out.2(.Generic, class(e1), class(e2))
+		      dr <- as(r, fullCl)
+		      ## FIXME: implement this:
+		      dr[ind.0(e1)] <- r0
+		  }
+	      }
+	      else { ## dsparseMatrix => lClass is "lsparse*"
 
 		  if(identical(r0, FALSE)) { ## things remain sparse
-		      if((Ar <- all(r)) || !any(r)) {
+		      if(!any(is.na(r)) && ((Ar <- all(r)) || !any(r))) {
 			  r <- new(lClass, Dim= dim(e1), Dimnames= dimnames(e1))
-			  if(Ar) # 'TRUE' instead of 'x': same sparsity:
+			  if(Ar) { # 'TRUE' instead of 'x': same sparsity:
+			      r@x <- rep.int(TRUE, length(e1@x))
 			      for(n in intersect(c("i","j","p"), slotNames(r)))
 				  slot(r, n) <- slot(e1, n)
+                          }
 			  ## else: all FALSE: keep empty 'r' matrix
-		      } else { # some TRUE, some FALSE: go via unique 'Tsparse'
+		      } else { # some TRUE, FALSE, NA : go via unique 'Tsparse'
 			  M <- asTuniq(e1)
-			  rx <- callGeneric(M@x, e2)
-                          ## FIXME! what if  any(is.na(rx))  ? !!!
-			  r <- new(class2(class(M), 'l'), # logical Tsparse
-				   i = M@i[rx], Dim = M@Dim,
-				   j = M@j[rx], Dimnames = M@Dimnames)
+			  nCl <- class2(class(M), 'l') # logical Tsparse
+			  r <- new(nCl)
+			  r@x <- callGeneric(M@x, e2)
+			  ## copy "the other slots" (important for "tr"/"sym"):
+			  ## "%w/o%" <- function(x,y) x[is.na(match(x, y))]
+			  sN <- slotNames(nCl)
+			  for(n in sN[is.na(match(sN, "x"))])
+			      slot(r, n) <- slot(M, n)
 			  if(is(e1, "CsparseMatrix"))
 			      r <- as(r, "CsparseMatrix")
 			  else if(is(e1, "RsparseMatrix"))
 			      r <- as(r, "RsparseMatrix")
 		      }
 		  } else {
-		      message(sprintf("sparse to dense coercion in '%s'",
-				      .Generic))
-                      r <- new(lClass, x = r,
-                               Dim = dim(e1), Dimnames = dimnames(e1))
+		      ## non sparse result
+		      message(sprintf("sparse to dense (%s) coercion in '%s'",
+				      lClass, .Generic))
+		      ## NOT YET:
+		      .bail.out.2(.Generic, class(e1), class(e2))
 
-                      ## NOT YET:
-                      .bail.out.2(.Generic, class(e1), class(e2))
+		      ## FIXME: implement this:
+		      r[ind.0(e1)] <- r0
 
-                      ## non sparse result
-
-                      ## FIXME: implement this:
-                      r[ind.0(e1)] <- r0
-                  }
+		      r <- new(lClass, x = r,
+			       Dim = dim(e1), Dimnames = dimnames(e1))
+		  }
 	      }
-              r
+	      r
 	  })
 
 ## "dMatrix <-> work with 'x' slot
