@@ -15,7 +15,7 @@ setAs("dgCMatrix", "dsCMatrix",
 ##       function(from) .Call(Csparse_to_Tsparse, from, FALSE))
 
 setAs("dsCMatrix", "dgTMatrix", # needed for image()
-      function(from) ## pre-Cholmod:
+      function(from) ## pre-Cholmod -- FIXME: get rid of
       .Call(dsCMatrix_to_dgTMatrix, from))
 
 setAs("dsCMatrix", "dgeMatrix",
@@ -30,6 +30,10 @@ setAs("matrix", "dsCMatrix",
 
 setAs("dsCMatrix", "lsCMatrix",
       function(from) new("lsCMatrix", i = from@i, p = from@p, uplo = from@uplo,
+                         x = as.logical(from@x),
+                         Dim = from@Dim, Dimnames = from@Dimnames))
+setAs("dsCMatrix", "nsCMatrix",
+      function(from) new("nsCMatrix", i = from@i, p = from@p, uplo = from@uplo,
                          Dim = from@Dim, Dimnames = from@Dimnames))
 
 setAs("dsCMatrix", "dgCMatrix",
@@ -83,12 +87,12 @@ setMethod("solve", signature(a = "dsCMatrix", b = "numeric"),
 ##          valueClass = "dgeMatrix")
 
 setMethod("chol", signature(x = "dsCMatrix", pivot = "missing"),
-          function(x, pivot, LINPACK) .Call(dsCMatrix_chol, x, FALSE),
-          valueClass = "dtCMatrix")
+	  function(x, pivot, ...) .Call(dsCMatrix_chol, x, FALSE),
+	  valueClass = "dtCMatrix")
 
 setMethod("chol", signature(x = "dsCMatrix", pivot = "logical"),
-          function(x, pivot, LINPACK) .Call(dsCMatrix_chol, x, pivot),
-          valueClass = "dtCMatrix")
+	  function(x, pivot, ...) .Call(dsCMatrix_chol, x, pivot),
+	  valueClass = "dtCMatrix")
 
 setMethod("Cholesky", signature(A = "dsCMatrix"),
           function(A, perm = TRUE, LDL = TRUE, super = FALSE, ...)
