@@ -4,11 +4,13 @@
 
 SEXP Csparse_validate(SEXP x)
 {
+    /* NB: we do *NOT* check a potential 'x' slot here, at all */
+    cholmod_sparse *chx = as_cholmod_sparse(x);
     SEXP pslot = GET_SLOT(x, Matrix_pSym),
 	islot = GET_SLOT(x, Matrix_iSym);
-    int j, ncol = length(pslot) - 1,
+    int j, k, ncol = length(pslot) - 1,
 	*dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
-	nrow, *xp = INTEGER(pslot),
+	nrow, sorted, *xp = INTEGER(pslot),
 	*xi = INTEGER(islot);
 
     nrow = dims[0];
@@ -17,15 +19,21 @@ SEXP Csparse_validate(SEXP x)
     if (xp[0] != 0)
 	return mkString(_("first element of slot p must be zero"));
     if (length(islot) != xp[ncol])
-	return mkString(_("last element of slot p must match length of slots i and x"));
-    for (j = 0; j < ncol; j++) {
-	if (xp[j] > xp[j+1])
-	    return mkString(_("slot p must be non-decreasing"));
-    }
+	return
+	    mkString(_("last element of slot p must match length of slots i and x"));
     for (j = 0; j < length(islot); j++) {
 	if (xi[j] < 0 || xi[j] >= nrow)
 	    return mkString(_("all row indices must be between 0 and nrow-1"));
     }
+    sorted = TRUE;
+    for (j = 0; j < ncol; j++) {
+	if (xp[j] > xp[j+1])
+	    return mkString(_("slot p must be non-decreasing"));
+	for (k = xp[j] + 1; k < xp[j + 1]; k++)
+	    if (xi[k] < xi[k - 1]) sorted = FALSE;
+    }
+    if (!sorted) cholmod_sort(chx, &c);
+    Free(chx);
     return ScalarLogical(1);
 }
 
@@ -35,10 +43,10 @@ SEXP Csparse_to_dense(SEXP x)
     cholmod_dense *chxd = cholmod_sparse_to_dense(chxs, &c);
 
     Free(chxs);
-    return chm_dense_to_SEXP(chxd, 1);
+    return chm_dense_to_SEXP(chxd, 1, Real_kind(x));
 }
 
-SEXP Csparse_to_logical(SEXP x, SEXP tri)
+SEXP Csparse_to_nz_pattern(SEXP x, SEXP tri)
 {
     cholmod_sparse *chxs = as_cholmod_sparse(x);
     cholmod_sparse
@@ -51,8 +59,8 @@ SEXP Csparse_to_logical(SEXP x, SEXP tri)
 	    -1 : 1;
 	diag = CHAR(asChar(GET_SLOT(x, Matrix_diagSym)));
     }
-    return chm_sparse_to_SEXP(chxcp, 1, uploT, diag,
-				GET_SLOT(x, Matrix_DimNamesSym));
+    return chm_sparse_to_SEXP(chxcp, 1, uploT, 0, diag,
+			      GET_SLOT(x, Matrix_DimNamesSym));
 }
 
 SEXP Csparse_to_matrix(SEXP x)
@@ -69,14 +77,16 @@ SEXP Csparse_to_Tsparse(SEXP x, SEXP tri)
 {
     cholmod_sparse *chxs = as_cholmod_sparse(x);
     cholmod_triplet *chxt = cholmod_sparse_to_triplet(chxs, &c);
-    int uploT = 0; char *diag = "";
+    int uploT = 0;
+    char *diag = "";
+    int Rkind = (chxs->xtype == CHOLMOD_REAL) ? Real_kind(x) : 0;
 
     Free(chxs);
     if (asLogical(tri)) {	/* triangular sparse matrices */
 	uploT = (*uplo_P(x) == 'U') ? -1 : 1;
 	diag = diag_P(x);
     }
-    return chm_triplet_to_SEXP(chxt, 1, uploT, diag,
+    return chm_triplet_to_SEXP(chxt, 1, uploT, Rkind, diag,
 			       GET_SLOT(x, Matrix_DimNamesSym));
 }
 
@@ -84,19 +94,21 @@ SEXP Csparse_to_Tsparse(SEXP x, SEXP tri)
 SEXP Csparse_symmetric_to_general(SEXP x)
 {
     cholmod_sparse *chx = as_cholmod_sparse(x), *chgx;
+    int Rkind = (chx->xtype == CHOLMOD_REAL) ? Real_kind(x) : 0;
 
     if (!(chx->stype))
-	error(_("Nonsymmetric matrix in Csparse_symmeteric_to_general"));
+	error(_("Nonsymmetric matrix in Csparse_symmetric_to_general"));
     chgx = cholmod_copy(chx, /* stype: */ 0, chx->xtype, &c);
     /* xtype: pattern, "real", complex or .. */
     Free(chx);
-    return chm_sparse_to_SEXP(chgx, 1, 0, "",
+    return chm_sparse_to_SEXP(chgx, 1, 0, Rkind, "",
 			      GET_SLOT(x, Matrix_DimNamesSym));
 }
 
 SEXP Csparse_transpose(SEXP x, SEXP tri)
 {
     cholmod_sparse *chx = as_cholmod_sparse(x);
+    int Rkind = (chx->xtype == CHOLMOD_REAL) ? Real_kind(x) : 0;
     cholmod_sparse *chxt = cholmod_transpose(chx, (int) chx->xtype, &c);
     SEXP dn = PROTECT(duplicate(GET_SLOT(x, Matrix_DimNamesSym))), tmp;
     int uploT = 0; char *diag = "";
@@ -110,7 +122,7 @@ SEXP Csparse_transpose(SEXP x, SEXP tri)
 	uploT = (*uplo_P(x) == 'U') ? -1 : 1;
 	diag = diag_P(x);
     }
-    return chm_sparse_to_SEXP(chxt, 1, uploT, diag, dn);
+    return chm_sparse_to_SEXP(chxt, 1, uploT, Rkind, diag, dn);
 }
 
 SEXP Csparse_Csparse_prod(SEXP a, SEXP b)
@@ -125,21 +137,21 @@ SEXP Csparse_Csparse_prod(SEXP a, SEXP b)
 		   duplicate(VECTOR_ELT(GET_SLOT(a, Matrix_DimNamesSym), 0)));
     SET_VECTOR_ELT(dn, 1,
 		   duplicate(VECTOR_ELT(GET_SLOT(b, Matrix_DimNamesSym), 1)));
-    return chm_sparse_to_SEXP(chc, 1, 0, "", dn);
+    return chm_sparse_to_SEXP(chc, 1, 0, 0, "", dn);
 }
 
 SEXP Csparse_dense_prod(SEXP a, SEXP b)
 {
     cholmod_sparse *cha = as_cholmod_sparse(a);
     cholmod_dense *chb = as_cholmod_dense(PROTECT(mMatrix_as_dgeMatrix(b)));
-    cholmod_dense *chc = 
+    cholmod_dense *chc =
 	cholmod_allocate_dense(cha->nrow, chb->ncol, cha->nrow, chb->xtype, &c);
     double alpha[] = {1,0}, beta[] = {0,0};
 
     cholmod_sdmult(cha, 0, alpha, beta, chb, chc, &c);
     Free(cha); Free(chb);
     UNPROTECT(1);
-    return chm_dense_to_SEXP(chc, 1);
+    return chm_dense_to_SEXP(chc, 1, 0);
 }
 
 SEXP Csparse_dense_crossprod(SEXP a, SEXP b)
@@ -153,7 +165,7 @@ SEXP Csparse_dense_crossprod(SEXP a, SEXP b)
     cholmod_sdmult(cha, 1, alpha, beta, chb, chc, &c);
     Free(cha); Free(chb);
     UNPROTECT(1);
-    return chm_dense_to_SEXP(chc, 1);
+    return chm_dense_to_SEXP(chc, 1, 0);
 }
 
 SEXP Csparse_crossprod(SEXP x, SEXP trans, SEXP triplet)
@@ -187,38 +199,41 @@ SEXP Csparse_crossprod(SEXP x, SEXP trans, SEXP triplet)
 					(tr) ? 1 : 0)));
     SET_VECTOR_ELT(dn, 1, duplicate(VECTOR_ELT(dn, 0)));
     UNPROTECT(1);
-    return chm_sparse_to_SEXP(chcp, 1, 0, "", dn);
+    return chm_sparse_to_SEXP(chcp, 1, 0, 0, "", dn);
 }
 
 SEXP Csparse_horzcat(SEXP x, SEXP y)
 {
     cholmod_sparse *chx = as_cholmod_sparse(x),
 	*chy = as_cholmod_sparse(y), *ans;
+    int Rkind = 0; /* only for "d" - FIXME */
 
     ans = cholmod_horzcat(chx, chy, 1, &c);
     Free(chx); Free(chy);
     /* FIXME: currently drops dimnames */
-    return chm_sparse_to_SEXP(ans, 1, 0, "", R_NilValue);
+    return chm_sparse_to_SEXP(ans, 1, 0, Rkind, "", R_NilValue);
 }
 
 SEXP Csparse_vertcat(SEXP x, SEXP y)
 {
     cholmod_sparse *chx = as_cholmod_sparse(x),
 	*chy = as_cholmod_sparse(y), *ans;
+    int Rkind = 0; /* only for "d" - FIXME */
 
     ans = cholmod_vertcat(chx, chy, 1, &c);
     Free(chx); Free(chy);
     /* FIXME: currently drops dimnames */
-    return chm_sparse_to_SEXP(ans, 1, 0, "", R_NilValue);
+    return chm_sparse_to_SEXP(ans, 1, 0, Rkind, "", R_NilValue);
 }
 
 SEXP Csparse_band(SEXP x, SEXP k1, SEXP k2)
 {
     cholmod_sparse *chx = as_cholmod_sparse(x), *ans;
+    int Rkind = (chx->xtype == CHOLMOD_REAL) ? Real_kind(x) : 0;
 
     ans = cholmod_band(chx, asInteger(k1), asInteger(k2), chx->xtype, &c);
     Free(chx);
-    return chm_sparse_to_SEXP(ans, 1, 0, "", R_NilValue);
+    return chm_sparse_to_SEXP(ans, 1, 0, Rkind, "", R_NilValue);
 }
 
 SEXP Csparse_diagU2N(SEXP x)
@@ -229,9 +244,10 @@ SEXP Csparse_diagU2N(SEXP x)
     cholmod_sparse *ans = cholmod_add(chx, eye, one, one, TRUE, TRUE, &c);
     int uploT = (strcmp(CHAR(asChar(GET_SLOT(x, Matrix_uploSym))), "U")) ?
 	-1 : 1;
+    int Rkind = (chx->xtype == CHOLMOD_REAL) ? Real_kind(x) : 0;
 
     Free(chx); cholmod_free_sparse(&eye, &c);
-    return chm_sparse_to_SEXP(ans, 1, uploT, "N",
+    return chm_sparse_to_SEXP(ans, 1, uploT, Rkind, "N",
 			      duplicate(GET_SLOT(x, Matrix_DimNamesSym)));
 }
 
@@ -240,6 +256,7 @@ SEXP Csparse_submatrix(SEXP x, SEXP i, SEXP j)
     cholmod_sparse *chx = as_cholmod_sparse(x);
     int rsize = (isNull(i)) ? -1 : LENGTH(i),
 	csize = (isNull(j)) ? -1 : LENGTH(j);
+    int Rkind = (chx->xtype == CHOLMOD_REAL) ? Real_kind(x) : 0;
 
     if (rsize >= 0 && !isInteger(i))
 	error(_("Index i must be NULL or integer"));
@@ -248,5 +265,5 @@ SEXP Csparse_submatrix(SEXP x, SEXP i, SEXP j)
     return chm_sparse_to_SEXP(cholmod_submatrix(chx, INTEGER(i), rsize,
 						INTEGER(j), csize,
 						TRUE, TRUE, &c),
-			      1, 0, "", R_NilValue);
+			      1, 0, Rkind, "", R_NilValue);
 }
