@@ -24,8 +24,6 @@
 #include "lgTMatrix.h"
 #include "lsCMatrix.h"
 #include "ltCMatrix.h"
-#include "lmer.h"
-#include "pedigree.h"
 #include "sparseQR.h"
 #include <R_ext/Rdynload.h>
 
@@ -45,7 +43,7 @@ static R_CallMethodDef CallEntries[] = {
     {"Csparse_horzcat", (DL_FUNC) &Csparse_horzcat, 2},
     {"Csparse_to_Tsparse", (DL_FUNC) &Csparse_to_Tsparse, 2},
     {"Csparse_to_dense", (DL_FUNC) &Csparse_to_dense, 1},
-    {"Csparse_to_logical", (DL_FUNC) &Csparse_to_logical, 2},
+    {"Csparse_to_nz_pattern", (DL_FUNC) &Csparse_to_nz_pattern, 2},
     {"Csparse_to_matrix", (DL_FUNC) &Csparse_to_matrix, 1},
     {"Csparse_submatrix", (DL_FUNC) &Csparse_submatrix, 3},
     {"Csparse_symmetric_to_general",
@@ -60,14 +58,12 @@ static R_CallMethodDef CallEntries[] = {
     {"LU_expand", (DL_FUNC) &LU_expand, 1},
     {"LU_validate", (DL_FUNC) &LU_validate, 1},
     {"Matrix_expand_pointers", (DL_FUNC) &Matrix_expand_pointers, 1},
-    {"Matrix_rWishart", (DL_FUNC) &Matrix_rWishart, 3},
     {"Matrix_writeHarwellBoeing", (DL_FUNC) &Matrix_writeHarwellBoeing, 3},
     {"Matrix_writeMatrixMarket", (DL_FUNC) &Matrix_writeMatrixMarket, 3},
     {"SVD_validate", (DL_FUNC) &SVD_validate, 1},
     {"Tsparse_validate", (DL_FUNC) &Tsparse_validate, 1},
     {"Tsparse_to_Csparse", (DL_FUNC) &Tsparse_to_Csparse, 2},
-    {"csc_check_column_sorting", (DL_FUNC) &csc_check_column_sorting, 1},
-    {"Zt_create", (DL_FUNC) &Zt_create, 2},
+/*     {"csc_check_column_sorting", (DL_FUNC) &csc_check_column_sorting, 1}, */
     {"compressed_to_dgTMatrix", (DL_FUNC) &compressed_to_dgTMatrix, 2},
     {"compressed_non_0_ij", (DL_FUNC) &compressed_non_0_ij, 2},
     {"dense_to_Csparse", (DL_FUNC) &dense_to_Csparse, 1},
@@ -138,6 +134,7 @@ static R_CallMethodDef CallEntries[] = {
     {"dtCMatrix_solve", (DL_FUNC) &dtCMatrix_solve, 1},
     {"dtCMatrix_matrix_solve", (DL_FUNC) &dtCMatrix_matrix_solve, 3},
     {"dtCMatrix_upper_solve", (DL_FUNC) &dtCMatrix_upper_solve, 1},
+    {"dtCMatrix_validate", (DL_FUNC) &dtCMatrix_validate, 1},
     {"dtTMatrix_as_dtrMatrix", (DL_FUNC) &dtTMatrix_as_dtrMatrix, 1},
     {"dtTMatrix_as_dgCMatrix", (DL_FUNC) &dtTMatrix_as_dgCMatrix, 1},
     {"dtTMatrix_validate", (DL_FUNC) &dtTMatrix_validate, 1},
@@ -159,64 +156,30 @@ static R_CallMethodDef CallEntries[] = {
     {"dtrMatrix_solve", (DL_FUNC) &dtrMatrix_solve, 1},
     {"dtrMatrix_validate", (DL_FUNC) &dtrMatrix_validate, 1},
     {"dup_mMatrix_as_dgeMatrix", (DL_FUNC) &dup_mMatrix_as_dgeMatrix, 1},
-    {"glmer_MCMCsamp", (DL_FUNC) &glmer_MCMCsamp, 5},
-    {"glmer_PQL", (DL_FUNC) &glmer_PQL, 1},
-    {"glmer_devLaplace", (DL_FUNC) &glmer_devLaplace, 2},
-    {"glmer_finalize", (DL_FUNC) &glmer_finalize, 1},
-    {"glmer_init", (DL_FUNC) &glmer_init, 1},
 
     {"lapack_qr", (DL_FUNC) &lapack_qr, 2},
 
     {"lcsc_to_matrix", (DL_FUNC) &lcsc_to_matrix, 1},
+    {"ncsc_to_matrix", (DL_FUNC) &ncsc_to_matrix, 1},
     {"lgCMatrix_validate", (DL_FUNC) &lgCMatrix_validate, 1},
     {"lgTMatrix_validate", (DL_FUNC) &lgTMatrix_validate, 1},
 
-    {"lspMatrix_as_lsyMatrix", (DL_FUNC) &lspMatrix_as_lsyMatrix, 1},
-    {"lsyMatrix_as_lspMatrix", (DL_FUNC) &lsyMatrix_as_lspMatrix, 1},
-    {"lsyMatrix_as_lgeMatrix", (DL_FUNC) &lsyMatrix_as_lgeMatrix, 1},
-    {"ltpMatrix_as_ltrMatrix", (DL_FUNC) &ltpMatrix_as_ltrMatrix, 1},
-    {"ltrMatrix_as_lgeMatrix", (DL_FUNC) &ltrMatrix_as_lgeMatrix, 1},
-    {"ltrMatrix_as_ltpMatrix", (DL_FUNC) &ltrMatrix_as_ltpMatrix, 1},
+    {"lspMatrix_as_lsyMatrix", (DL_FUNC) &lspMatrix_as_lsyMatrix, 2},
+    {"lsyMatrix_as_lspMatrix", (DL_FUNC) &lsyMatrix_as_lspMatrix, 2},
+    {"lsyMatrix_as_lgeMatrix", (DL_FUNC) &lsyMatrix_as_lgeMatrix, 2},
+    {"ltpMatrix_as_ltrMatrix", (DL_FUNC) &ltpMatrix_as_ltrMatrix, 2},
+    {"ltrMatrix_as_lgeMatrix", (DL_FUNC) &ltrMatrix_as_lgeMatrix, 2},
+    {"ltrMatrix_as_ltpMatrix", (DL_FUNC) &ltrMatrix_as_ltpMatrix, 2},
 
     {"lsCMatrix_validate", (DL_FUNC) &lsCMatrix_validate, 1},
     {"ltCMatrix_validate", (DL_FUNC) &ltCMatrix_validate, 1},
     {"lsq_dense_Chol", (DL_FUNC) &lsq_dense_Chol, 2},
     {"lsq_dense_QR", (DL_FUNC) &lsq_dense_QR, 2},
-
-    {"mer_ECMEsteps", (DL_FUNC) &mer_ECMEsteps, 3},
-    {"mer_MCMCsamp", (DL_FUNC) &mer_MCMCsamp, 5},
-    {"mer_coef", (DL_FUNC) &mer_coef, 2},
-    {"mer_coefGets", (DL_FUNC) &mer_coefGets, 3},
-    {"mer_create", (DL_FUNC) &mer_create, 10},
-    {"mer_dtCMatrix", (DL_FUNC) &mer_dtCMatrix, 1},
-    {"mer_dtCMatrix_inv", (DL_FUNC) &mer_dtCMatrix_inv, 1},
-    {"mer_factor", (DL_FUNC) &mer_factor, 1},
-    {"mer_fitted", (DL_FUNC) &mer_fitted, 1},
-    {"mer_fixef", (DL_FUNC) &mer_fixef, 1},
-    {"mer_gradComp", (DL_FUNC) &mer_gradComp, 1},
-    {"mer_gradient", (DL_FUNC) &mer_gradient, 2},
-    {"mer_hat_trace", (DL_FUNC) &mer_hat_trace, 1},
-    {"mer_hat_trace2", (DL_FUNC) &mer_hat_trace2, 1},
-    {"mer_initial", (DL_FUNC) &mer_initial, 1},
-    {"mer_isNested", (DL_FUNC) &mer_isNested, 1},
-    {"mer_postVar", (DL_FUNC) &mer_postVar, 1},
-    {"mer_ranef", (DL_FUNC) &mer_ranef, 1},
-    {"mer_secondary", (DL_FUNC) &mer_secondary, 1},
-    {"mer_sigma", (DL_FUNC) &mer_sigma, 2},
-    {"mer_simulate", (DL_FUNC) &mer_simulate, 2},
-    {"mer_update_ZXy", (DL_FUNC) &mer_update_ZXy, 1},
-    {"mer_update_y", (DL_FUNC) &mer_update_y, 2},
-
-/*     {"matrix_to_csc", (DL_FUNC) &matrix_to_csc, 1}, */
-    {"pedigree_chol", (DL_FUNC) &pedigree_chol, 2},
-/*     {"ssc_transpose", (DL_FUNC) &ssc_transpose, 1}, */
     {"sparseQR_validate", (DL_FUNC) &sparseQR_validate, 1},
     {"sparseQR_qty", (DL_FUNC) &sparseQR_qty, 3},
     {"sparseQR_coef", (DL_FUNC) &sparseQR_coef, 2},
     {"sparseQR_resid_fitted", (DL_FUNC) &sparseQR_resid_fitted, 3},
-    {"tsc_to_dgTMatrix", (DL_FUNC) &tsc_to_dgTMatrix, 1},
-/*     {"tsc_transpose", (DL_FUNC) &tsc_transpose, 1}, */
-    {"tsc_validate", (DL_FUNC) &tsc_validate, 1},
+/*     {"tsc_to_dgTMatrix", (DL_FUNC) &tsc_to_dgTMatrix, 1}, */
     {"triangularMatrix_validate", (DL_FUNC) &triangularMatrix_validate, 1},
     {"symmetricMatrix_validate", (DL_FUNC) &symmetricMatrix_validate, 1},
     {NULL, NULL, 0}
@@ -230,54 +193,50 @@ R_init_Matrix(DllInfo *dll)
 {
     R_registerRoutines(dll, NULL, CallEntries, NULL, NULL);
     R_useDynamicSymbols(dll, FALSE);
+
+    R_RegisterCCallable("Matrix", "as_cholmod_dense", (DL_FUNC)as_cholmod_dense);
+    R_RegisterCCallable("Matrix", "as_cholmod_factor", (DL_FUNC)as_cholmod_factor);
+    R_RegisterCCallable("Matrix", "as_cholmod_sparse", (DL_FUNC)as_cholmod_sparse);
+    R_RegisterCCallable("Matrix", "chm_factor_to_SEXP", (DL_FUNC)chm_factor_to_SEXP);
+
+    R_RegisterCCallable("Matrix", "cholmod_aat", (DL_FUNC)cholmod_aat);
+    R_RegisterCCallable("Matrix", "cholmod_add", (DL_FUNC)cholmod_add);
+    R_RegisterCCallable("Matrix", "cholmod_allocate_dense", (DL_FUNC)cholmod_allocate_dense);
+    R_RegisterCCallable("Matrix", "cholmod_allocate_sparse", (DL_FUNC)cholmod_allocate_sparse);
+    R_RegisterCCallable("Matrix", "cholmod_analyze", (DL_FUNC)cholmod_analyze);
+    R_RegisterCCallable("Matrix", "cholmod_copy", (DL_FUNC)cholmod_copy);
+    R_RegisterCCallable("Matrix", "cholmod_copy_dense", (DL_FUNC)cholmod_copy_dense);
+    R_RegisterCCallable("Matrix", "cholmod_copy_factor", (DL_FUNC)cholmod_copy_factor);
+    R_RegisterCCallable("Matrix", "cholmod_copy_sparse", (DL_FUNC)cholmod_copy_sparse);
+    R_RegisterCCallable("Matrix", "cholmod_factor_to_sparse", (DL_FUNC)cholmod_factor_to_sparse);
+    R_RegisterCCallable("Matrix", "cholmod_factorize", (DL_FUNC)cholmod_factorize);
+    R_RegisterCCallable("Matrix", "cholmod_finish", (DL_FUNC)cholmod_finish);
+    R_RegisterCCallable("Matrix", "cholmod_free_dense", (DL_FUNC)cholmod_free_dense);
+    R_RegisterCCallable("Matrix", "cholmod_free_factor", (DL_FUNC)cholmod_free_sparse);
+    R_RegisterCCallable("Matrix", "cholmod_free_sparse", (DL_FUNC)cholmod_free_sparse);
+    R_RegisterCCallable("Matrix", "cholmod_nnz", (DL_FUNC)cholmod_nnz);
+    R_RegisterCCallable("Matrix", "cholmod_sdmult", (DL_FUNC)cholmod_sdmult);
+    R_RegisterCCallable("Matrix", "cholmod_solve", (DL_FUNC)cholmod_solve);
+    R_RegisterCCallable("Matrix", "cholmod_speye", (DL_FUNC)cholmod_speye);
+    R_RegisterCCallable("Matrix", "cholmod_spsolve", (DL_FUNC)cholmod_spsolve);
+    R_RegisterCCallable("Matrix", "cholmod_start", (DL_FUNC)cholmod_start);
+    R_RegisterCCallable("Matrix", "cholmod_transpose", (DL_FUNC)cholmod_transpose);
+
+    R_RegisterCCallable("Matrix", "dpoMatrix_chol", (DL_FUNC)dpoMatrix_chol);
+    R_RegisterCCallable("Matrix", "numeric_as_chm_dense", (DL_FUNC)numeric_as_chm_dense);
+
     cholmod_start(&c);
-    Matrix_DSym = install("D");
+
     Matrix_DimNamesSym = install("Dimnames");
     Matrix_DimSym = install("Dim");
-    Matrix_GpSym = install("Gp");
-    Matrix_LSym = install("L");
-    Matrix_OmegaSym = install("Omega");
-    Matrix_ParentSym = install("Parent");
-    Matrix_RXXSym = install("RXX");
-    Matrix_RZXSym = install("RZX");
-    Matrix_RZXinvSym = install("RZXinv");
-    Matrix_XSym = install("X");
-    Matrix_XtXSym = install("XtX");
-    Matrix_XtySym = install("Xty");
-    Matrix_ZZpOSym = install("ZZpO");
-    Matrix_ZZxSym = install("ZZx");
-    Matrix_ZtSym = install("Zt");
-    Matrix_ZtXSym = install("ZtX");
-    Matrix_ZtZSym = install("ZtZ");
-    Matrix_ZtySym = install("Zty");
-    Matrix_bVarSym = install("bVar");
-    Matrix_callSym = install("call");
-    Matrix_cnamesSym = install("cnames");
-    Matrix_devCompSym = install("devComp");
-    Matrix_devianceSym = install("deviance");
     Matrix_diagSym = install("diag");
     Matrix_factorSym = install("factors");
-    Matrix_familySym = install("family");
-    Matrix_fixefSym = install("fixef");
-    Matrix_flistSym = install("flist");
-    Matrix_gradCompSym = install("gradComp");
     Matrix_iSym = install("i");
     Matrix_jSym = install("j");
-    Matrix_methodSym = install("method");
-    Matrix_ncSym = install("nc");
     Matrix_pSym = install("p");
     Matrix_permSym = install("perm");
-    Matrix_rXySym = install("rXy");
-    Matrix_rZySym = install("rZy");
-    Matrix_ranefSym = install("ranef");
-    Matrix_statusSym = install("status");
     Matrix_uploSym = install("uplo");
-    Matrix_useScaleSym = install("useScale");
-    Matrix_wrkresSym = install("wrkres");
-    Matrix_wtsSym = install("wts");
     Matrix_xSym = install("x");
-    Matrix_ySym = install("y");
-    Matrix_zSym = install("z");
 }
 
 void R_unload_Matrix(DllInfo *dll)
